% ファイル名：gpd_fir.m
% FIRフィルタのフィルタ係数を読み込み，振幅・位相・群遅延特性を表示する．
% 正規化周波数は，MATLAB signal processing toolboxの正規化周波数では
% なく，サンプリング周波数で正規化した通常の正規化周波数としてグラフを
% 描画している．ファイルから読み込みするフィルタ係数値（インパルス応答値）は，
% 以下の形式
%
%   3       先頭行はフィルタ係数の個数N（インパルス応答長）
%   0.33    そのあとN個のフィルタ係数（インパルス応答値）が並ぶ
%   0.33
%   0.33
%
% 2021.11.17 Y.Yamada その都度作成してした内容をまとめて作成
% 2021.11.22 Y.Yamada unwrap( )を使用するように
% 2024. 4.26 Y.Yamada figureウインドウにタイトルをつけた
% 2024. 5.10 Y.Yamada フィルタ係数読み込みを関数に
% 2024. 5.13 Y.Yamada Webページで公開 https://dsp-lab.net/release/
%
clear; close all;
%%%%%%%% 計算の設定
DFTsize = 16384; % 振幅・位相・群遅延特性計算時のDFT点数
IsPhaseUnwrapping = true; % true or false, trueなら，位相アンラップする
PhaseInRadian = true;     % true or false, trueなら位相の値をradianで表示する
                          % false なら，degreeで表示する
%%%%%%%% グラフ描画
YdB = 1;               % 振幅特性を±YdBの目盛りで拡大表示
ThresholdLength = 65;  % インパルス応答長がこの値以下の場合，インパルス応答値のグラフをstem( )で表示する
width = 500;           % グラフの横幅（ドット数） MATLABのデフォルトは560
height = 300;          % グラフの縦（ドット数） MATLABのデフォルトは480
left = 400; bottom = 300; % グラフの描画位置（左下隅の左からと下からの座標）
left_slip = 30; bottom_slip = 20; % グラフの描画位置をちょっぴり移動

% フィルタ係数ファイル名取得
[fname, dpath] = uigetfile({'*.txt','txtファイル(*.txt)';...
                            '*.*','すべてのファイル(*.*)'}...
                                 ,'読み込みするFIRフィルタ係数ファイルの選択');    
if isequal(fname,0) || isequal(dpath,0)
    return;
end
fname = fullfile(dpath, fname);
[impN, coef] = FirCoefRead(fname);
% impN:インパルス応答長, coef: impN個のインパルス応答値（フィルタ係数値）

% plot impulse response
index = 0:1:impN-1;
fg = figure('Name','インパルス応答 Impulse response');
fg.Position(3:4) = [width height]; movegui([left bottom]);
if impN <= ThresholdLength
    stem(index, coef);  
else
    plot(index, coef);
end
xlim([0 impN-1]);
xlabel('Time in sample');
ylabel('Magnitude');

%%%% frequency response of FIR digital filter
[response, nfreq] = freqz(coef,1,DFTsize,1);

% amplitude response
fg = figure('Name','振幅特性 Amplitude response');
fg.Position(3:4) = [width height]; 
left = left+left_slip; bottom = bottom-bottom_slip; movegui([left bottom]);
plot( nfreq, 20.0*log10(abs(response)) );
xlabel('Normalized frequency');     
ylabel('Amplitude in dB');

% amplitude response（拡大）
fg = figure('Name','振幅特性の拡大 Amplitude response (magnified)');
fg.Position(3:4) = [width height]; 
left = left+left_slip; bottom = bottom-bottom_slip; movegui([left bottom]);
plot( nfreq, 20.0*log10(abs(response)) );
%xlim([0 0.5]);
ylim([-YdB YdB]);
xlabel('Normalized frequency')       
ylabel('Amplitude in dB')

% phase response
if isequal(PhaseInRadian,true)
    if isequal(IsPhaseUnwrapping,true)
        ph = unwrap(angle(response)); 
    else
        ph = angle(response);
    end
else  % degreeに変換して表記
    if isequal(IsPhaseUnwrapping,true)
        ph = 180.0 * unwrap(angle(response))/pi;
    else
        ph = 180.0 * angle(response)/pi;
    end
end
fg = figure('Name','位相特性 Phase response'); 
left = left+left_slip; bottom = bottom-bottom_slip; movegui([left bottom]);
fg.Position(3:4) = [width height];
plot(nfreq,ph);
xlabel('Normalized frequency')
if isequal(PhaseInRadian,true)
    ylabel('Phase in radian')
else
    ylabel('Phase in degree')
end

% group delay
[gd,nfreq] = grpdelay(coef,1,DFTsize,1);
fg = figure('Name','群遅延特性 Group delay');
fg.Position(3:4) = [width height]; 
left = left+left_slip; bottom = bottom-bottom_slip; movegui([left bottom]);
plot(nfreq,gd);
xlabel('Normalized frequency')
ylabel('Group delay in sample')
ylim([0 impN]);

%%%% ここから下は，関数定義
function [N, coef] = FirCoefRead(FileName) 
% 指定ファイル名からFIRフィルタ係数値を読み込む
fp = fopen(FileName,'r');
N = fscanf(fp,'%d',1);    % インパルス応答長
coef = fscanf(fp,'%f',N); % フィルタ係数
fclose(fp);
end